# -*- encoding: utf-8 -*-
require 'omniauth-oauth2'

module OmniAuth
  module Strategies
    class Yhd < OmniAuth::Strategies::OAuth2
      option :name, 'yhd'
      option :client_options, {
                 site: "https://member.yhd.com", # 
        authorize_url: "/login/authorize.do",
            token_url: "/login/token.do"
      }

      option :token_params, {
        parse: :json
      }

      uid do
        raw_info['storeId']
      end

      info do
        {
          nickname: raw_info['storeName']
        }
      end

      extra do
        {
          raw_info: raw_info
        }
      end

      def client
        ::OAuth2::YhdClient.new(options.client_id, options.client_secret, deep_symbolize(options.client_options))
      end

      def raw_info
        access_token.options[:mode] = :query
        @raw_info ||= ::OAuth2::Response.new(conn.post('/app/api/rest/router', signed_params), parse: :json).parsed['response']['storeMerchantStoreInfo'] rescue {}
      end

      private

      def conn
        @conn ||= Faraday.new(url: "http://openapi.yhd.com") do |b|
          b.response :logger
          b.adapter  :typhoeus
        end
      end
      
      def params
        {
          method: 'yhd.store.get',
          sessionKey: access_token.token,
          appKey: client.id,
          timestamp: timestamp,
          format: 'json',
          ver: '1.0'
        }
      end

      def signed_params
         rand = client.secret + params.sort.flatten.join + client.secret
        token = Digest::MD5.hexdigest(rand)
        params.merge(sign: token)# 數字簽名
      end

      def timestamp # 时间戳
        Time.now.strftime("%Y-%m-%d %H:%M:%S")
      end
    end
  end
end

OmniAuth.config.add_camelization "yhd", "Yhd"